unit FileList;

{$IMPORTEDDATA OFF}

interface

type
  TFileList = class(TObject)
  public
    Cur: Integer;
    Count: Integer;
    Items: array of WideString;
    Positions: array of Cardinal; // for archive plugin
    constructor Create;
    procedure Add(const Path: WideString; Position: Cardinal = 0);
    procedure AddDir(const Path: WideString);
    procedure Clear;
    procedure Delete(Index: Integer);
    procedure Sort;
  end;

implementation

uses ShlwapiSub, NLib, NSys;

constructor TFileList.Create;
begin
  Clear;
end;

procedure TFileList.Add(const Path: WideString; Position: Cardinal = 0);
begin
  if Count = Length(Items) then begin
    SetLength(Items, Count + Count div 4);
    SetLength(Positions, Count + Count div 4);
  end;
  Items[Count] := Path;
  Positions[Count] := Position;
  Inc(Count);
end;

procedure TFileList.AddDir(const Path: WideString);
var
  dir: TDir;
begin
  with TListDir.Create(Path) do
    try
      while Each(dir) do
        if not (dmDirectory in dir.Mode) then
          Add(JoinPath([Path, dir.Name]));
    finally
      Free;
    end;
end;

procedure TFileList.Clear;
begin
  Count := 0;
  SetLength(Items, 32);
  SetLength(Positions, 32);
end;

procedure TFileList.Delete(Index: Integer);
var
  i: Integer;
begin
  if (Index < 0) or (Index >= Count) then
    Exit;
  Dec(Count);
  for i := Index to Count - 1 do
    Items[i] := Items[i+1];
end;

function StrCmpLogicalWrapperW(const A: WideString; const B: WideString): Integer;
begin
  Result := StrCmpLogicalW(PWideChar(A), PWideChar(B));
end;

procedure TFileList.Sort;
begin
  if Count = 0 then Exit;

  SetLength(Items, Count);
  if Assigned(StrCmpLogicalW) then
    NLib.Sort(Items, StrCmpLogicalWrapperW)
  else
    NLib.Sort(Items, StrCollW);
end;

end.
